<?php
/**
 * @package     CleverReach
 * @subpackage  cleverreach_loader
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class PlgSystemCleverreach_Loader extends JPlugin
{
	/**
	 * Method that is called after initialization..
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	public function onAfterInitialise()
	{
		$this->registerCoreAndServices();
	}

	/**
	 * Method to register CleverReach Core library.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	private function registerCoreAndServices()
	{
		if (!$this->coreExists() || !$this->componentEnabled())
		{
			return;
		}

		$this->registerComponentClasses();

		if (JVERSION < 3.7)
		{
			$this->registerCoreClasses(JPATH_LIBRARIES . '/cleverreach_core/src', JPATH_LIBRARIES . '/cleverreach_core/src');
		}
		else
		{
			JLoader::registerNamespace('CleverReach', JPATH_LIBRARIES . '/cleverreach_core/src', false, true, 'psr4');
		}

		$this->registerServices();
	}

	/**
	 * Check if core exists, before loading it.
	 *
	 * @return boolean Returns true if core exists, otherwise returns false.
	 *
	 * @since 1.0.0
	 */
	private function coreExists()
	{
		$core = JLibraryHelper::getLibrary('cleverreach_core', true);

		if ($core === null)
		{
			return false;
		}

		return $core->enabled;
	}

	/**
	 * Check if component is enabled, before registering services.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function componentEnabled()
	{
		return (bool) JComponentHelper::getComponent('com_cleverreach', true)->enabled;
	}

	/**
	 * Registers all classes in CleverReach component.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function registerComponentClasses()
	{
		//  Tables

		JLoader::register(
			'CleverreachTableProcess',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/table/process.php'
		);
		JLoader::register(
			'CleverreachTableConfig',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/table/config.php'
		);
		JLoader::register(
			'CleverreachTableQueue',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/table/queue.php'
		);

		//  Repositories

		JLoader::register(
			'CleverreachRepositoryBase',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/base.php'
		);
		JLoader::register(
			'CleverreachRepositoryProcess',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/process.php'
		);
		JLoader::register(
			'CleverreachRepositoryConfig',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/config.php'
		);
		JLoader::register(
			'CleverreachRepositoryQueue',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/queue.php'
		);
		JLoader::register(
			'CleverreachRepositoryUser',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/user.php'
		);
		JLoader::register(
			'CleverreachRepositoryArticle',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/repository/article.php'
		);

		//  Controllers

		JLoader::register(
			'CleverreachControllerBase',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/controller/base.php'
		);
		JLoader::register(
			'CleverreachControllerCallback',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/controller/callback.php'
		);
		JLoader::register(
			'CleverreachControllerCheckstatus',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/controller/checkstatus.php'
		);

		JLoader::register(
			'CleverreachControllerAsyncprocessstarter',
			JPATH_BASE . '/components/com_cleverreach/controller/asyncprocessstarter.php'
		);
		JLoader::register(
			'CleverreachControllerConfig',
			JPATH_BASE . '/components/com_cleverreach/controller/config.php'
		);
		JLoader::register(
			'CleverreachControllerArticlesearch',
			JPATH_BASE . '/components/com_cleverreach/controller/articlesearch.php'
		);
		JLoader::register(
			'CleverreachControllerEventhandler',
			JPATH_BASE . '/components/com_cleverreach/controller/eventhandler.php'
		);


		//  Utilities

		JLoader::register(
			'CleverreachUtilityHelper',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/utility/helper.php'
		);
		JLoader::register(
			'CleverreachUtilityTaskqueue',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/utility/taskqueue.php'
		);
		JLoader::register(
			'CleverreachUtilityArticleseachprovider',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/utility/articlesearchprovider.php'
		);

		//  Views

		JLoader::register(
			'CleverreachViewWelcome',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/welcome/view.html.php'
		);
		JLoader::register(
			'CleverreachViewInitialsyncconfig',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/initialsyncconfig/view.html.php'
		);
		JLoader::register(
			'CleverreachViewInitialsync',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/initialsync/view.html.php'
		);
		JLoader::register(
			'CleverreachViewDashboard',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/dashboard/view.html.php'
		);
		JLoader::register(
			'CleverreachViewTokenexpired',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/tokenexpired/view.html.php'
		);
		JLoader::register(
			'CleverreachViewCorenotloaded',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/corenotloaded/view.html.php'
		);

		//  Tasks

		JLoader::register(
			'CleverreachServiceSyncSetDefaultNewsletterStatusTask',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/sync/setdefaultnewsletterstatustask.php'
		);

		JLoader::register(
			'CleverreachServiceSyncCustominitialsynctask',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/sync/custominitialsynctask.php'
		);
	}

	/**
	 * Method that scans every file in CleverReach integration core and registers it as a class
	 * for all Joomla! CMS versions below 3.7 which don't support PSR-4
	 *
	 * @param   string $basePath Base path that remains the same.
	 * @param   string $path     Path that changes recursively.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function registerCoreClasses($basePath, $path)
	{
		$handle = @opendir($path);

		while (false !== ($file = readdir($handle)))
		{
			if ($file === '.' || $file === '..')
			{
				continue;
			}

			if (is_dir("$path/$file"))
			{
				$this->registerCoreClasses($basePath, "$path/$file");
			}
			else
			{
				$base      = str_replace($basePath, 'CleverReach', $path);
				$classPath = str_replace('/', '\\', $base);
				JLoader::register(
					$classPath . '\\' . str_replace('.php', '', $file),
					$path . '/' . $file
				);
			}
		}

		closedir($handle);
	}

	/**
	 * Registers core and implemented services.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function registerServices()
	{
		JLoader::register(
			'CleverreachServiceInfrastructureLogger',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/logger.php'
		);
		JLoader::register(
			'CleverreachServiceInfrastructureHttpclient',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/httpclient.php'
		);
		JLoader::register(
			'CleverreachServiceInfrastructureAsyncprocessstarter',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/asyncprocessstarter.php'
		);
		JLoader::register(
			'CleverreachServiceInfrastructureConfigrepository',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/configrepository.php'
		);
		JLoader::register(
			'CleverreachServiceInfrastructureConfig',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/config.php'
		);
		JLoader::register(
			'CleverreachServiceInfrastructureTaskqueuestorage',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/infrastructure/taskqueuestorage.php'
		);
		JLoader::register(
			'CleverreachServiceBusinesslogicAttributes',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/businesslogic/attributes.php'
		);
		JLoader::register(
			'CleverreachServiceBusinesslogicRecipients',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/service/businesslogic/recipients.php'
		);

		JLoader::register(
			'CleverreachUtilityInitializer',
			JPATH_ADMINISTRATOR . '/components/com_cleverreach/utility/initializer.php'
		);

		CleverreachUtilityInitializer::registerServices();
	}

	/**
	 * Method that is called after plugin install.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	public function onExtensionAfterInstall()
	{
		$this->registerCoreAndServices();
	}
}
