<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

defined('_JEXEC') or die('Restricted access');

use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\TaskExecution\QueueItem;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachController extends JControllerLegacy
{
	const JS_AND_CSS_VERSION = 2;

	/**
	 * Method to display a view.
	 *
	 * @param   boolean      $cachable  If true, the view output will be cached
	 * @param   array | bool $urlparams An array of safe URL parameters and their variable types, for valid values see {@link JFilterInput::clean()}.
	 *
	 * @return  void
	 *
	 * @throws Exception
	 *
	 * @since   1.0.0
	 */
	public function display($cachable = false, $urlparams = false)
	{
		if (!$this->requiredClassesExist())
		{
			$templateName = 'corenotloaded';

			JLoader::register(
				'CleverreachViewCorenotloaded',
				JPATH_ADMINISTRATOR . '/components/com_cleverreach/view/corenotloaded/view.html.php'
			);
		}
		elseif (!$this->isAuthTokenValid())
		{
			$templateName = 'welcome';
		}
		elseif (!$this->isInitialSyncConfigured())
		{
			$templateName = 'initialsyncconfig';
		}
		elseif ($this->isInitialSyncInProgress())
		{
			$templateName = 'initialsync';
		}
		elseif (!$this->checkIfTokenRefreshed())
		{
			$templateName = 'tokenexpired';
		}
		else
		{
			$templateName = 'dashboard';
		}

		$view = $this->getView($templateName, 'html');
		$view->setLayout('modal');
		$view->display();
	}

	/**
	 * Checks if required core and loader classes exist in current Joomla! installation.
	 *
	 * @return boolean
	 *   If classes exist returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	private function requiredClassesExist()
	{
		return class_exists('CleverreachUtilityHelper') && CleverreachUtilityHelper::requiredClassesExist();
	}

	/**
	 * Checks if there is a valid auth token in configuration.
	 *
	 * @return boolean
	 *
	 * @throws \InvalidArgumentException
	 *
	 * @since 1.0.0
	 */
	public function isAuthTokenValid()
	{
		return CleverreachUtilityHelper::isAuthTokenValid();
	}

	/**
	 * Checks if user is already configured initial sync.
	 *
	 * @return boolean
	 *   If sync configured returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	private function isInitialSyncConfigured()
	{
		return CleverreachUtilityHelper::isInitialSyncConfigured();
	}

	/**
	 * Checks if initial sync is in progress and enqueues the task if initial sync hasn't been started.
	 *
	 * @return boolean
	 *
	 * @throws \InvalidArgumentException
	 *
	 * @since 1.0.0
	 */
	public function isInitialSyncInProgress()
	{
		return CleverreachUtilityHelper::isInitialSyncInProgress(true);
	}

	/**
	 * Checks if token has been refreshed.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function checkIfTokenRefreshed()
	{
		/** @var CleverreachServiceInfrastructureTaskqueuestorage $queueService */
		$queueService      = ServiceRegister::getService(Queue::CLASS_NAME);
		$exchangeTokenTask = $queueService->findLatestByType('ExchangeAccessTokenTask');

		return CleverreachUtilityHelper::checkRefreshToken()
			|| ($exchangeTokenTask !== null
				&& !in_array($exchangeTokenTask->getStatus(), array(QueueItem::COMPLETED, QueueItem::FAILED), true)
			);
	}
}
