<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachRepositoryConfig extends CleverreachRepositoryBase
{
	/**
	 * ConfigRepository constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		parent::__construct();

		$this->tableName = CleverreachTableConfig::CONFIG_TABLE;
		$this->idColumn  = 'key';
	}

	/**
	 * Creates or updated configuration for given key.
	 *
	 * @param   string $key    Key of the configuration option.
	 * @param   mixed  $value  Value of the configuration option.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function updateValue($key, $value)
	{
		if ($this->valueExists($key))
		{
			return $this->update(array('value' => $value), array('key' => $key));
		}

		return $this->insert(array('key' => $key, 'value' => $value));
	}

	/**
	 * Gets configuration value for given key.
	 *
	 * @param   string $key Key of configuration option.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function getValue($key)
	{
		$config = $this->findByPk($key);

		if (empty($config))
		{
			return null;
		}

		if (is_array($config))
		{
			return $config['value'];
		}

		return $config->value;
	}

	/**
	 * Checks if a value exists in configuration.
	 *
	 * @param   string $key Key of configuration option.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	private function valueExists($key)
	{
		$config = $this->findByPk($key);

		return !empty($config);
	}
}
