<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Exposed\Runnable;
use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\TaskExecution\Exceptions\ProcessStorageGetException;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachRepositoryProcess extends CleverreachRepositoryBase
{
	/**
	 * ProcessRepository constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		parent::__construct();

		$this->tableName = CleverreachTableProcess::PROCESS_TABLE;
	}

	/**
	 * Saves process.
	 *
	 * @param   string   $guid   ID of the process.
	 * @param   Runnable $runner Runnable object.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function save($guid, Runnable $runner)
	{
		$this->insert(array('id' => $guid, 'runner' => serialize($runner)));
	}

	/**
	 * Gets runner for provided GUID.
	 *
	 * @param   string $guid ID of the process.
	 *
	 * @return Runnable
	 *
	 * @throws ProcessStorageGetException
	 *
	 * @since 1.0.0
	 */
	public function getRunner($guid)
	{
		$process = $this->findByPk($guid);

		if ($process === null)
		{
			throw new ProcessStorageGetException('Process runner with guid ' . $guid . ' does not exist.');
		}

		return unserialize($process->runner);
	}

	/**
	 * Deletes process for provided GUID.
	 *
	 * @param   string $guid ID of the process.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function deleteProcess($guid)
	{
		$process = $this->findByPk($guid);

		if ($process === null)
		{
			Logger::logError('Could not delete process with guid ' . $guid);
		}
		else
		{
			$this->delete(array('id' => $guid));
		}
	}
}
