<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Entity\RecipientAttribute;
use CleverReach\BusinessLogic\Interfaces\Attributes;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\ServiceRegister;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceBusinesslogicAttributes implements Attributes
{
	/**
	 * @var array
	 *
	 * @since 1.0.0
	 */
	private $attributes;

	/**
	 * CleverreachServiceBusinesslogicAttributes constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct()
	{
		$profileEnabled = JPluginHelper::isEnabled('user', 'profile');

		$this->attributes = array(
			'email'          => 'COM_CONTACT_SHOW_EMAIL_ADDRESS_LABEL',
			'firstname'      => 'COM_CLEVERREACH_FIRST_NAME',
			'lastname'       => 'COM_CLEVERREACH_LAST_NAME',
			'shop'           => 'CMS',
			'customernumber' => 'COM_CLEVERREACH_CUSTOMER_NUMBER',
			'language'       => 'language',
			'newsletter'     => 'PLG_USER_CLEVERREACH_FIELDSET_NAME',
			'birthday'       => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_DOB_LABEL' : 'birthday',
			'street'         => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_ADDRESS1_LABEL' : 'street',
			'zip'            => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_POSTAL_CODE_LABEL' : 'zip',
			'state'          => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_REGION_LABEL' : 'state',
			'city'           => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_CITY_LABEL' : 'city',
			'country'        => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_COUNTRY_LABEL' : 'country',
			'phone'          => $profileEnabled ? 'PLG_USER_PROFILE_FIELD_PHONE_LABEL' : 'phone',
		);
	}

	/**
	 * Get attributes from integration with translated params in system language.
	 *
	 * It should set name, description, preview_value and default_value for each attribute available in system.
	 *
	 * @return RecipientAttribute[]
	 *   List of available attributes in the system.
	 */
	public function getAttributes()
	{
		$language = JFactory::getLanguage();
		/** @var string $tag Language tag (ex. en-GB) */
		$tag        = $this->getConfigService()->getUserLanguage();
		$locale     = explode('-', $tag);
		$tag        = in_array($locale[0], array('en', 'de', 'it', 'fr', 'es')) ? $tag : 'en-GB';
		$attributes = array();
		$language->load('com_contact', JPATH_ADMINISTRATOR, $tag);
		$language->load('plg_user_profile', JPATH_ADMINISTRATOR, $tag);
		$language->load('cleverreach', JPATH_ADMINISTRATOR, $tag);
		$language->load('com_cleverreach', JPATH_ADMINISTRATOR, $tag);

		foreach ($this->attributes as $name => $key)
		{
			$recipientAttribute = new RecipientAttribute($name);
			$recipientAttribute->setDescription($language->_($key));

			$attributes[] = $recipientAttribute;
		}

		return $attributes;
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}
}
