<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\Configuration;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureConfig extends Configuration
{
	const INTEGRATION_NAME = 'Joomla';
	const INTEGRATION_LIST_NAME = 'Joomla! CMS';
	const CLIENT_ID = 'FTs5oh374n';
	const CLIENT_SECRET = 'roFbXelTLohXTo1CWWlp90iC6GU9OYxf';

	/**
	 * Retrieves integration name.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getIntegrationName()
	{
		return self::INTEGRATION_NAME;
	}

	/**
	 * Returns integration list name.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getIntegrationListName()
	{
		return self::INTEGRATION_LIST_NAME;
	}

	/**
	 * Retrieves client ID.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getClientId()
	{
		return self::CLIENT_ID;
	}

	/**
	 * Retrieves client secret.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getClientSecret()
	{
		return self::CLIENT_SECRET;
	}

	/**
	 * Returns the name of the queue.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getQueueName()
	{
		return 'Joomla';
	}

	/**
	 * Sets site name in the configuration.
	 *
	 * @param   string $siteName Name of the site.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function setSiteName($siteName)
	{
		$this->getConfigRepository()->set('CLEVERREACH_SITE_NAME', $siteName);
	}


	/**
	 * Returns site name from the configuration.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getSiteName()
	{
		return $this->getConfigRepository()->get('CLEVERREACH_SITE_NAME');
	}

	/**
	 * Sets user language in the configuration.
	 *
	 * @param   string $language Language tag.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function setUserLanguage($language)
	{
		$this->getConfigRepository()->set('CLEVERREACH_USER_LANGUAGE', $language);
	}

	/**
	 * Returns user language from the configuration.
	 *
	 * @return integer|string
	 *
	 * @since 1.0.0
	 */
	public function getUserLanguage()
	{
		return $this->getConfigRepository()->get('CLEVERREACH_USER_LANGUAGE');
	}

	/**
	 * Returns URL to the event handler.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public function getCrEventHandlerURL()
	{
		return JUri::root() . JRouter::getInstance('Site')
				->build('?option=com_cleverreach&task=hookHandler')
				->toString(array('query', 'fragment'));
	}

	/**
	 * Checks if initial sync is configured.
	 *
	 * @return boolean If initial sync is configured, returns true, otherwise false.
	 *
	 * @since 1.0.0
	 */
	public function isConfiguredInitialSync()
	{
		return (bool) $this->getConfigRepository()->get('CLEVERREACH_CONFIGURED_INITIAL_SYNC');
	}

	/**
	 * Sets flag that indicate whether initial sync is configured or not.
	 *
	 * @param   bool $value Indicator whether initial sync is configured or not.
	 *
	 * @throws Exception When module is not installed.
	 *
	 * @since 1.0.0
	 */
	public function setConfiguredInitialSync($value = true)
	{
		$this->getConfigRepository()->set('CLEVERREACH_CONFIGURED_INITIAL_SYNC', $value);
	}

	/**
	 * Return true if recipient is active by default, otherwise false.
	 *
	 * @return boolean Default subscription behavior.
	 *
	 * @since 1.0.0
	 */
	public function getDefaultRecipientStatus()
	{
		return (bool) $this->getConfigRepository()->get('CLEVERREACH_DEFAULT_RECIPIENT_STATUS');
	}

	/**
	 * Sets default recipient status.
	 *
	 * @param   bool $status Default subscription behavior.
	 *
	 * @throws Exception When module is not installed.
	 *
	 * @since 1.0.0
	 */
	public function setDefaultRecipientStatus($status)
	{
		$this->getConfigRepository()->set('CLEVERREACH_DEFAULT_RECIPIENT_STATUS', $status);
	}

	/**
	 * Returns last update version of the plugin.
	 *
	 * @return string Last update version of the plugin.
	 *
	 * @since 1.0.0
	 */
	public function getLastUpdateVersion()
	{
		return $this->getConfigRepository()->get('CLEVERREACH_LAST_UPDATE_VERSION');
	}

	/**
	 * Sets last update version of the plugin.
	 *
	 * @param string $version Last update version of the plugin
	 *
	 * @since 1.0.0
	 */
	public function setLastUpdateVersion($version)
	{
		$this->getConfigRepository()->set('CLEVERREACH_LAST_UPDATE_VERSION', $version);
	}
}
