<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\ConfigRepositoryInterface;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureConfigrepository implements ConfigRepositoryInterface
{
	/**
	 * @var CleverreachRepositoryConfig
	 *
	 * @since 1.0.0
	 */
	private $configRepository;

	/**
	 * Gets the value with this key from the configuration.
	 *
	 * @param   string $key Key of configuration option.
	 *
	 * @return string | int
	 *
	 * @since 1.0.0
	 */
	public function get($key)
	{
		return $this->getConfigRepository()->getValue($key);
	}

	/**
	 * Stores the value with a given key in the configuration.
	 *
	 * @param   string $key   Key of configuration option.
	 * @param   mixed  $value Value of configuration option.
	 *
	 * @return mixed
	 *
	 * @since 1.0.0
	 */
	public function set($key, $value)
	{
		$result = $this->getConfigRepository()->updateValue($key, $value);

		return $result !== false;
	}

	/**
	 * Returns configuration repository.
	 *
	 * @return CleverreachRepositoryConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigRepository()
	{
		if ($this->configRepository === null)
		{
			$this->configRepository = new CleverreachRepositoryConfig;
		}

		return $this->configRepository;
	}
}
