<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\LoggerAdapter;
use CleverReach\Infrastructure\Logger\Configuration;
use CleverReach\Infrastructure\Logger\LogData;
use CleverReach\Infrastructure\Logger\Logger;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceInfrastructureLogger implements LoggerAdapter
{

	/**
	 * Gets web location of log file.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	public static function getLogLocation()
	{
		$config = new JConfig;

		return $config->log_path . '/cleverreach-' . date('Y-m-d') . '.log';
	}

	/**
	 * Logs message in system.
	 *
	 * @param   LogData $data Data to be logged.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public function logMessage($data)
	{
		/** @var Configuration $configService */
		$configService = Configuration::getInstance();
		$minLogLevel   = $configService->getMinLogLevel();
		$logLevel      = $data->getLogLevel();

		if ($logLevel > $minLogLevel)
		{
			return;
		}

		$level = 'info';

		switch ($logLevel)
		{
			case Logger::ERROR:
				$level = 'error';
				break;
			case Logger::WARNING:
				$level = 'warning';
				break;
			case Logger::DEBUG:
				$level = 'debug';
				break;
		}

		$log = fopen(self::getLogLocation(), 'ab+');
		fwrite(
			$log,
			"[$level][" . $data->getTimestamp() . '][' .
			$data->getComponent() . '][' . $data->getUserAccount() . '] ' . $data->getMessage() . "\n"
		);
		fclose($log);
	}
}
