<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Sync\InitialSyncTask;
use CleverReach\Infrastructure\TaskExecution\Task;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachServiceSyncCustominitialsynctask extends InitialSyncTask
{
	/**
	 * Creates a subtask of initial sync task.
	 *
	 * @param   string $taskKey Key of the task
	 *
	 * @return Task
	 *
	 * @since 1.0.0
	 */
	protected function createSubTask($taskKey)
	{
		if ($taskKey === 'CleverreachServiceSyncSetDefaultNewsletterStatusTask')
		{
			return new CleverreachServiceSyncSetDefaultNewsletterStatusTask;
		}

		return parent::createSubTask($taskKey);
	}

	/**
	 * Sets tasks for the last group in initial sync (recipients) to the list of sub tasks.
	 *
	 * @param   array $subTasks Array of subtasks.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	protected function setRecipientSyncTasks(array &$subTasks)
	{
		$subTasks[$this->getDefaultNewsletterStatusTaskName()] = 10;
		$subTasks[$this->getRecipientSyncTaskName()] = $this->getConfigService()->isProductSearchEnabled() ? 50 : 55;
	}

	/**
	 * Gets the key for the recipient sync task.
	 *
	 * @return string
	 *
	 * @since 1.0.0
	 */
	private function getDefaultNewsletterStatusTaskName()
	{
		return CleverreachServiceSyncSetDefaultNewsletterStatusTask::getClassName();
	}
}
