<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Proxy;
use CleverReach\BusinessLogic\Interfaces\Proxy as ProxyInterface;
use CleverReach\Infrastructure\Interfaces\DefaultLoggerAdapter;
use CleverReach\Infrastructure\Interfaces\Required\AsyncProcessStarter;
use CleverReach\Infrastructure\Interfaces\Required\ConfigRepositoryInterface;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\Interfaces\Required\HttpClient;
use CleverReach\Infrastructure\Interfaces\Required\ShopLoggerAdapter;
use CleverReach\Infrastructure\Logger\DefaultLogger;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\TaskExecution\Queue;
use CleverReach\Infrastructure\TaskExecution\TaskRunner;
use CleverReach\Infrastructure\TaskExecution\TaskRunnerStatusStorage;
use CleverReach\Infrastructure\Interfaces\Exposed\TaskRunnerStatusStorage as TaskRunnerStatusStorageInterface;
use CleverReach\Infrastructure\TaskExecution\TaskRunnerWakeup;
use CleverReach\Infrastructure\Interfaces\Exposed\TaskRunnerWakeup as TaskRunnerWakeUpInterface;
use CleverReach\Infrastructure\Utility\GuidProvider;
use CleverReach\Infrastructure\Utility\TimeProvider;
use CleverReach\Infrastructure\Interfaces\Required\TaskQueueStorage;
use CleverReach\BusinessLogic\Interfaces\Attributes;
use CleverReach\BusinessLogic\Interfaces\Recipients;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachUtilityInitializer
{
	/**
	 * Registers all services.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	public static function registerServices()
	{
		try
		{
			/** ********* Default CORE services ****************** */
			ServiceRegister::registerService(
				TimeProvider::CLASS_NAME,
				function () {
					return new TimeProvider;
				}
			);

			ServiceRegister::registerService(
				Queue::CLASS_NAME,
				function () {
					return new Queue;
				}
			);

			ServiceRegister::registerService(
				ProxyInterface::CLASS_NAME,
				function () {
					return new Proxy;
				}
			);

			ServiceRegister::registerService(
				TaskRunnerWakeUpInterface::CLASS_NAME,
				function () {
					return new TaskRunnerWakeup;
				}
			);

			ServiceRegister::registerService(
				TaskRunner::CLASS_NAME,
				function () {
					return new TaskRunner;
				}
			);

			ServiceRegister::registerService(
				GuidProvider::CLASS_NAME,
				function () {
					return new GuidProvider;
				}
			);

			ServiceRegister::registerService(
				DefaultLoggerAdapter::CLASS_NAME,
				function () {
					return new DefaultLogger;
				}
			);

			ServiceRegister::registerService(
				TaskRunnerStatusStorageInterface::CLASS_NAME,
				function () {
					return new TaskRunnerStatusStorage;
				}
			);

			/** ********* Implemented services ****************** */

			ServiceRegister::registerService(
				HttpClient::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureHttpclient;
				}
			);

			ServiceRegister::registerService(
				ShopLoggerAdapter::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureLogger;
				}
			);

			ServiceRegister::registerService(
				AsyncProcessStarter::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureAsyncprocessstarter;
				}
			);

			ServiceRegister::registerService(
				ConfigRepositoryInterface::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureConfigrepository;
				}
			);

			ServiceRegister::registerService(
				Configuration::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureConfig;
				}
			);

			ServiceRegister::registerService(
				TaskQueueStorage::CLASS_NAME,
				function () {
					return new CleverreachServiceInfrastructureTaskqueuestorage;
				}
			);

			ServiceRegister::registerService(
				Attributes::CLASS_NAME,
				function () {
					return new CleverreachServiceBusinesslogicAttributes;
				}
			);

			ServiceRegister::registerService(
				Recipients::CLASS_NAME,
				function () {
					return new CleverreachServiceBusinesslogicRecipients;
				}
			);
		}
		catch (InvalidArgumentException $exception)
		{
		}
	}
}
