<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\Infrastructure\ServiceRegister;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @property array $config   Configuration values
 * @property array $progressItems
 *
 * @since       1.0.0
 */
class CleverreachViewInitialsync extends JViewLegacy
{
	/**
	 * Display initial sync view.
	 *
	 * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	public function display($tpl = null)
	{
		$this->setInitialSyncScreenParams();
		$this->addToolBar();

		parent::display($tpl);
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	protected function addToolBar()
	{
		JToolbarHelper::title(Text::_('COM_CLEVERREACH_TOOLBAR_TITLE'), ' icon-envelope');
	}

	/**
	 * Sets parameters for initial sync screen.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setInitialSyncScreenParams()
	{
		$statusCheckUrl = CleverreachUtilityHelper::getInitialSyncUrl();

		$this->config = array(
			'statusCheckUrl'  => $statusCheckUrl,
			'integrationName' => $this->getConfigService()->getIntegrationName(),
		);

		$prefix = 'COM_CLEVERREACH_INITIAL_SYNCHRONISATION_SCREEN_';

		$this->progressItems = array(
			'subscriber_list' => Text::_("{$prefix}CREATE_SUBSCRIBER_LIST"),
			'add_fields'      => Text::_("{$prefix}ADD_FIELDS"),
			'recipient_sync'  => sprintf(Text::_("{$prefix}IMPORT_SUBSCRIBERS"), $this->config['integrationName']),
		);
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}
}