<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;
use Joomla\CMS\Language\Text;
use CleverReach\BusinessLogic\Interfaces\Proxy;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @property array  $config   Configuration values
 * @property string $prefix
 *
 * @since       1.0.0
 */
class CleverreachViewTokenexpired extends JViewLegacy
{
	/**
	 * Display token expired view.
	 *
	 * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	public function display($tpl = null)
	{
		$this->setTokenExpiredScreenParams();
		$this->addToolBar();

		parent::display();
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	protected function addToolBar()
	{
		JToolbarHelper::title(Text::_('COM_CLEVERREACH_TOOLBAR_TITLE'), ' icon-envelope');
	}

	/**
	 * Sets parameters for token expired screen.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setTokenExpiredScreenParams()
	{
		$redirectUrl  = CleverreachUtilityHelper::getRefreshAuthRedirectUrl();
		$proxy        = ServiceRegister::getService(Proxy::CLASS_NAME);
		$localeList   = JFactory::getLanguage()->getLocale();
		$locale       = explode('_', $localeList[0]);
		$userInfo     = $this->getConfigService()->getUserInfo();

		$this->prefix = 'COM_CLEVERREACH_TOKEN_EXPIRED_SCREEN_';
		$this->config = array(
			'clientId'        => $userInfo['id'],
			'integrationName' => $this->getConfigService()->getIntegrationName(),
			'checkStatusUrl'  => CleverreachUtilityHelper::getUserInfoUrl(),
			'authUrl'         => $proxy->getAuthUrl($redirectUrl, '', array('lang' => $locale[0])),
			'wakeupUrl'       => CleverreachUtilityHelper::getWakeupUrl(),
		);
	}

	/**
	 * Returns configuration service.
	 *
	 * @return CleverreachServiceInfrastructureConfig
	 *
	 * @since 1.0.0
	 */
	private function getConfigService()
	{
		return ServiceRegister::getService(Configuration::CLASS_NAME);
	}
}