<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\BusinessLogic\Interfaces\Proxy;
use CleverReach\Infrastructure\ServiceRegister;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @property array $config   Configuration values
 *
 * @since       1.0.0
 */
class CleverreachViewWelcome extends JViewLegacy
{
	/**
	 * Display welcome view.
	 *
	 * @param   string $tpl The name of the template file to parse; automatically searches through the template paths.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	public function display($tpl = null)
	{
		$this->setWelcomeScreenParams();
		$this->addToolBar();

		parent::display();
	}

	/**
	 * Checks if cURL is enabled on the server.
	 *
	 * @return boolean
	 *
	 * @since 1.0.0
	 */
	public function isCurlEnabled()
	{
		return extension_loaded('curl');
	}

	/**
	 * Add the page title and toolbar.
	 *
	 * @return  void
	 *
	 * @since 1.0.0
	 */
	protected function addToolBar()
	{
		JToolbarHelper::title(Text::_('COM_CLEVERREACH_TOOLBAR_TITLE'), ' icon-envelope');
	}

	/**
	 * Sets parameters for welcome screen.
	 *
	 * @return void
	 *
	 * @since 1.0.0
	 */
	private function setWelcomeScreenParams()
	{
		$redirectUrl  = CleverreachUtilityHelper::getAuthRedirectUrl();
		$registerData = base64_encode(json_encode(CleverreachUtilityHelper::getRegisterData()));
		$proxy        = ServiceRegister::getService(Proxy::CLASS_NAME);
		$localeList   = JFactory::getLanguage()->getLocale();
		$locale       = explode('_', $localeList[0]);

		$this->config = array(
			'checkStatusUrl' => CleverreachUtilityHelper::getUserInfoUrl(),
			'authUrl'        => $proxy->getAuthUrl($redirectUrl, $registerData, array('lang' => $locale[0])),
			'wakeupUrl'      => CleverreachUtilityHelper::getWakeupUrl(),
		);
	}
}