<?php
/**
 * @package     CleverReach
 * @subpackage  com_cleverreach
 * @copyright   Copyright (C) 2019 CleverReach GmbH & Co. KG
 * @license     GNU General Public License version 2 or later
 * @since       1.0.0
 */

use CleverReach\Infrastructure\Logger\Logger;
use CleverReach\Infrastructure\ServiceRegister;
use CleverReach\Infrastructure\Interfaces\Required\Configuration;

defined('_JEXEC') or die;

/**
 * @package     CleverReach
 *
 * @since       1.0.0
 */
class CleverreachControllerAsyncprocessstarter extends CleverreachControllerBase
{
	/**
	 * Runs process defined by guid request parameter.
	 *
	 * @param   \JInput $input The application input object
	 *
	 * @return void
	 *
	 * @throws TaskRunnerStatusStorageUnavailableException
	 * @throws Exception
	 *
	 * @since 1.0.0
	 */
	public function run($input)
	{
		if (!JComponentHelper::getComponent('com_cleverreach', true)->enabled)
		{
			/** @var CleverreachServiceInfrastructureConfig $config */
			$config = ServiceRegister::getService(Configuration::CLASS_NAME);
			$config->setTaskRunnerStatus('', null);
			exit();
		}

		if (!$this->isPost())
		{
			$this->redirect404();
		}

		$guid              = $this->getParam('guid', $input);
		$processRepository = new CleverreachRepositoryProcess;

		try
		{
			/** @var \CleverReach\Infrastructure\TaskExecution\TaskRunner $runner */
			$runner = $processRepository->getRunner($guid);
			$runner->run();
			$processRepository->deleteProcess($guid);
		}
		catch (\Exception $e)
		{
			Logger::logError($e->getMessage(), 'Integration');
		}

		exit();
	}
}
